from collective.feedback.controlpanels.settings import ICollectiveFeedbackSettings
from collective.feedback.interfaces import ICollectiveFeedbackStore
from plone import api
from plone.protect.interfaces import IDisableCSRFProtection
from plone.restapi.deserializer import json_body
from plone.restapi.services import Service
from zExceptions import BadRequest
from zope.component import getUtility
from zope.interface import alsoProvides

import re


class FeedbackAdd(Service):
    """
    Service for add feedback to object
    """

    store = ICollectiveFeedbackStore

    def reply(self):
        alsoProvides(self.request, IDisableCSRFProtection)
        form_data = json_body(self.request)
        self.validate_form(form_data=form_data)
        data = self.extract_data(form_data=form_data)
        tool = getUtility(self.store)
        try:
            res = tool.add(data)
        except ValueError as e:
            self.request.response.setStatus(500)
            return dict(
                error=dict(
                    type="InternalServerError",
                    message=getattr(e, "message", e.__str__()),
                )
            )

        if res:
            return self.reply_no_content()

        self.request.response.setStatus(500)
        return dict(
            error=dict(
                type="InternalServerError",
                message="Unable to add. Contact site manager.",
            )
        )

    def validate_form(self, form_data):
        """
        check all required fields and parameters
        """
        for field in ["vote", "content"]:
            value = form_data.get(field, "")
            if not value:
                raise BadRequest("Campo obbligatorio mancante: {}".format(field))

    def looks_like_path(self, string):
        return bool(re.match(r"^(/|/[^\s<>:\"|?*]+.*)$", string))

    def extract_data(self, form_data):
        path = form_data.pop("content")
        if self.looks_like_path(path):
            portal = api.portal.get()
            contextual_path = "/" + portal.id + path
            context = api.content.get(path=contextual_path)
            if not context:
                raise BadRequest(f"Object with path {contextual_path} not found.")

            form_data.update({"uid": context.UID()})
            form_data.update({"title": context.Title()})
        else:
            allowed_view = api.portal.get_registry_record(
                "allowed_feedback_view",
                interface=ICollectiveFeedbackSettings,
                default=False,
            )
            if path not in allowed_view:
                raise BadRequest(f"View non consentita: {path}")

            form_data.update({"title": path})

        return form_data



# -*- coding: utf-8 -*-
from collective.feedback.controlpanels.settings import ICollectiveFeedbackSettings
from collective.feedback.controlpanels.settings import (
    ICollectiveFeedbackSettingsControlpanel,
)
from plone.restapi.controlpanels import RegistryConfigletPanel
from zope.component import adapter
from zope.interface import implementer
from zope.interface import Interface


@adapter(Interface, Interface)
@implementer(ICollectiveFeedbackSettingsControlpanel)
class CollectiveFeedbackSettings(RegistryConfigletPanel):
    schema = ICollectiveFeedbackSettings
    configlet_id = "CollectiveFeedbackSettings"
    configlet_category_id = "Products"
    schema_prefix = None



# -*- coding: utf-8 -*-
from collective.feedback.interfaces import ICollectiveFeedbackStore
from collective.feedback.testing import RESTAPI_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession
from zope.component import getUtility

import transaction
import unittest


class TestAdd(unittest.TestCase):
    layer = RESTAPI_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        self.portal_url_path = self.portal.absolute_url_path()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        api.user.create(
            email="memberuser@example.com",
            username="memberuser",
            password="secret!!",
        )

        self.document = api.content.create(
            title="Document", container=self.portal, type="Document"
        )
        api.content.transition(obj=self.document, transition="publish")

        self.private_document = api.content.create(
            title="restricted document", container=self.portal, type="Document"
        )
        transaction.commit()

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        self.anon_api_session = RelativeSession(self.portal_url)
        self.anon_api_session.headers.update({"Accept": "application/json"})

        self.url = "{}/@feedback-add".format(self.portal_url)
        self.document_path = self.document.absolute_url_path().replace(
            self.portal.absolute_url_path(), ""
        )
        self.private_document_path = self.private_document.absolute_url_path().replace(
            self.portal.absolute_url_path(), ""
        )

    def tearDown(self):
        self.api_session.close()
        self.anon_api_session.close()

    def test_correctly_update_data(self):
        self.anon_api_session.post(
            self.url,
            json={
                "vote": 3,
                "comment": "i disagree",
                "honey": "",
                "content": self.document_path,
            },
        )
        self.anon_api_session.post(
            self.url,
            json={
                "vote": 2,
                "comment": "i disagree",
                "honey": "",
                "content": self.document_path,
            },
        )
        transaction.commit()
        tool = getUtility(ICollectiveFeedbackStore)
        feedbacks = tool.search()

        self.assertEqual(len(feedbacks), 2)

        self.api_session.patch(
            self.portal_url + "/@feedback-list",
            json={str(feedbacks[0].intid): {"read": True}},
        )
        transaction.commit()

        self.assertTrue(tool.get(feedbacks[0].intid).attrs.get("read"))

    def test_unknown_id(self):
        self.anon_api_session.post(
            self.url,
            json={
                "vote": 3,
                "comment": "i disagree",
                "honey": "",
                "content": self.document_path,
            },
        )
        transaction.commit()

        tool = getUtility(ICollectiveFeedbackStore)
        feedbacks = tool.search()

        self.assertEqual(len(feedbacks), 1)

        resp = self.api_session.patch(
            self.portal_url + "/@feedback-list",
            json={"1111111111": {"read": True}},
        )

        transaction.commit()

        self.assertEqual(resp.status_code, 404)

    def test_bad_id(self):
        self.anon_api_session.post(
            self.url,
            json={
                "vote": 3,
                "comment": "i disagree",
                "honey": "",
                "content": self.document_path,
            },
        )
        transaction.commit()

        tool = getUtility(ICollectiveFeedbackStore)
        feedbacks = tool.search()

        self.assertEqual(len(feedbacks), 1)

        resp = self.api_session.patch(
            self.portal_url + "/@feedback-list",
            json={"fffffffff": {"read": True}},
        )

        transaction.commit()

        self.assertEqual(resp.status_code, 400)



from collective.feedback.interfaces import ICollectiveFeedbackStore
from plone.protect.interfaces import IDisableCSRFProtection
from plone.restapi.deserializer import json_body
from plone.restapi.services import Service
from zExceptions import BadRequest
from zExceptions import NotFound
from zope.component import getUtility
from zope.interface import alsoProvides


class FeedbacListkUpdate(Service):
    """
    Service for update feedback to object, you can only update `read` field
    """

    def __init__(self, context, request):
        super().__init__(context, request)

    def reply(self):
        alsoProvides(self.request, IDisableCSRFProtection)

        tool = getUtility(ICollectiveFeedbackStore)

        form_data = self.extract_data(json_body(self.request))

        for id, value in form_data.items():
            comment = tool.get(id)

            if comment.get("error", "") == "NotFound":
                raise NotFound()

            try:
                tool.update(id, value)
            except ValueError as e:
                self.request.response.setStatus(500)
                return dict(
                    error=dict(
                        type="InternalServerError",
                        message=getattr(e, "message", e.__str__()),
                    )
                )

        return form_data

    def extract_data(self, form_data):
        data = {}

        for id, value in form_data.items():
            try:
                self.validate_data(value)
                data[int(id)] = {"read": value.get("read")}
            except ValueError:
                raise BadRequest(f"Bad id={id} format provided")

        return data

    def validate_data(self, data):
        """
        check all required fields and parameters
        """
        for field in ["read"]:
            value = data.get(field, None)
            if value is None:
                raise BadRequest("Campo obbligatorio mancante: {}".format(field))



# -*- coding: utf-8 -*-
"""Installer for the collective.feedback package."""

from setuptools import find_packages
from setuptools import setup


long_description = "\n\n".join(
    [
        open("README.rst").read(),
        open("CONTRIBUTORS.rst").read(),
        open("CHANGES.rst").read(),
    ]
)


setup(
    name="collective.feedback",
    version="1.2.0",
    description="Feedback mechanism integration for io-comune",
    long_description=long_description,
    # Get more from https://pypi.org/classifiers/
    classifiers=[
        "Development Status :: 1 - Planning",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: Addon",
        "Framework :: Plone :: 5.2",
        "Framework :: Plone :: 6.0",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords="Python Plone CMS",
    author="RedTurtle Technology",
    author_email="sviluppoplone@redturtle.it",
    url="https://github.com/collective/collective.feedback",
    project_urls={
        "PyPI": "https://pypi.org/project/collective.feedback/",
        "Source": "https://github.com/collective/collective.feedback",
        "Tracker": "https://github.com/collective/collective.feedback/issues",
        # 'Documentation': 'https://collective.feedback.readthedocs.io/en/latest/',
    },
    license="GPL version 2",
    packages=find_packages("src", exclude=["ez_setup"]),
    namespace_packages=["collective"],
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.7",
    install_requires=[
        "setuptools",
        "souper.plone",
        "collective.honeypot>=2.1",
    ],
    extras_require={
        "test": [
            "gocept.pytestlayer",
            "plone.app.testing",
            "plone.restapi[test]",
            "pytest-cov",
            "pytest-plone>=0.2.0",
            "pytest-docker",
            "pytest-mock",
            "pytest",
            "zest.releaser[recommended]",
            "zestreleaser.towncrier",
            "pytest-mock",
            "requests-mock",
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    [console_scripts]
    update_locale = collective.feedback.locales.update:update_locale
    """,
)



from collective.feedback.interfaces import ICollectiveFeedbackStore
from plone.protect.interfaces import IDisableCSRFProtection
from plone.restapi.deserializer import json_body
from plone.restapi.services import Service
from zExceptions import BadRequest
from zExceptions import NotFound
from zope.component import getUtility
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.publisher.interfaces import IPublishTraverse


@implementer(IPublishTraverse)
class FeedbackUpdate(Service):
    """
    Service for update feedback to object, you can only update `read` field
    """

    def __init__(self, context, request):
        super().__init__(context, request)
        self.params = []

    def publishTraverse(self, request, id):
        # Consume any path segments after /@users as parameters
        self.params.append(id)
        return self

    def reply(self):
        alsoProvides(self.request, IDisableCSRFProtection)

        tool = getUtility(ICollectiveFeedbackStore)

        if self.params:
            try:
                id = int(self.params[0])
            except ValueError:
                raise BadRequest(f"Bad id={self.params[0]} format provided")

            comment = tool.get(id)

            if comment.get("error", "") == "NotFound":
                raise NotFound()

        form_data = json_body(self.request)

        self.validate_form(form_data=form_data)

        form_data = self.extract_data(form_data)

        try:
            res = tool.update(id, form_data)
        except ValueError as e:
            self.request.response.setStatus(500)
            return dict(
                error=dict(
                    type="InternalServerError",
                    message=getattr(e, "message", e.__str__()),
                )
            )

        if res is None:
            return self.reply_no_content()

        self.request.response.setStatus(500)

        return dict(
            error=dict(
                type="InternalServerError",
                message="Unable to add. Contact site manager.",
            )
        )

    def extract_data(sefl, form_data):
        return {"read": form_data.get("read")}

    def validate_form(self, form_data):
        """
        check all required fields and parameters
        """
        for field in ["read"]:
            value = form_data.get(field, None)
            if value is None:
                raise BadRequest("Campo obbligatorio mancante: {}".format(field))



# -*- coding: utf-8 -*-
from collective.feedback import _
from plone.app.registry.browser.controlpanel import ControlPanelFormWrapper
from plone.app.registry.browser.controlpanel import RegistryEditForm
from plone.restapi.controlpanels.interfaces import IControlpanel
from zope.interface import Interface
from zope.schema import List
from zope.schema import TextLine


class ICollectiveFeedbackSettingsControlpanel(IControlpanel):
    """ """


class ICollectiveFeedbackSettings(Interface):
    allowed_feedback_view = List(
        title=_(
            "allowed_feedback_view_label",
            default="Vista su cui è possibile lasciare dei feedback",
        ),
        required=True,
        default=[
            "Login/Logout",
            "Ricerca",
            "Search",
            "Le mie prenotazioni",
            "My bookings",
            "Prenotazione appuntamento",
            "Appointment booking",
            "Export appuntamenti",
            "Export bookings",
            "Area personale",
            "Personal area",
            "Area operatore",
            "Operator area",
        ],
        value_type=TextLine(),
    )


class CollectiveFeedbackControlPanelForm(RegistryEditForm):
    schema = ICollectiveFeedbackSettings
    id = "collective-feedback-control-panel"
    label = _("Impostazioni Collective Feedback")


class CollectiveFeedbackControlPanelView(ControlPanelFormWrapper):
    """ """

    form = CollectiveFeedbackControlPanelForm
